package rocks.zipcode.calcskin;
import javafx.application.Application;
import javafx.beans.binding.Bindings;
import javafx.beans.property.*;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.*;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import java.util.HashMap;
import java.util.Map;

public class CalcSkin extends Application {

    public static void main(String[] args){
        launch(args);
    }
    private static final String[][] template = {
            { "7", "8", "9", "/" },
            { "4", "5", "6", "*" },
            { "1", "2", "3", "-" },
            { "0", "c", "=", "+" },
            { "√", "^", "~", "²"}
    };

    private final Map<String, Button> accelerators = new HashMap<>();

    private DoubleProperty previousValue = new SimpleDoubleProperty();
    private DoubleProperty currentValue = new SimpleDoubleProperty();
    private CalcEngine calcEngine = new CalcEngine();

    private enum Cal { operation, add, subtract, multiply, divide, squareroot, square, exponent, invert}

    private Cal first   = Cal.operation;
    private Cal second = Cal.operation;

    public static void launchCalc(String... args) {
        launch(args);
    }

    @Override
    public void start(Stage stage) {
        final TextField screen  = createScreen();
        final TilePane  buttons = createButtons();

        stage.setTitle("Calc");
        stage.initStyle(StageStyle.UTILITY);
        stage.setResizable(false);
        stage.setScene(new Scene(createLayout(screen, buttons)));
        stage.show();
    }

    private VBox createLayout(TextField screen, TilePane buttons) {
        final VBox layout = new VBox(20);
        layout.setAlignment(Pos.CENTER);
        layout.setStyle("-fx-background-color: silver; -fx-padding: 20; -fx-font-size: 20;");
        layout.getChildren().setAll(screen, buttons);
        handleAccelerators(layout);
        screen.prefWidthProperty().bind(buttons.widthProperty());
        return layout;
    }

    private void handleAccelerators(VBox layout) {
        layout.addEventFilter(KeyEvent.KEY_PRESSED, new EventHandler<KeyEvent>() {
            @Override
            public void handle(KeyEvent keyEvent) {
                Button activated = accelerators.get(keyEvent.getText());
                if (activated != null) {
                    activated.fire();
                }
            }
        });
    }

    private TextField createScreen() {
        final TextField screen = new TextField();
        screen.setStyle("-fx-background-color: aquamarine;");
        screen.setAlignment(Pos.CENTER_RIGHT);
        screen.setEditable(false);
        screen.textProperty().bind(Bindings.format("%.0f", currentValue));
        return screen;
    }

    private TilePane createButtons() {
        TilePane buttons = new TilePane();
        buttons.setVgap(7);
        buttons.setHgap(7);
        buttons.setPrefColumns(template[0].length);
        for (String[] r: template) {
            for (String s: r) {
                buttons.getChildren().add(createButton(s));
            }
        }
        return buttons;
    }

    private Button createButton(final String s) {
        Button button = makeStandardButton(s);

        if (s.matches("[0-9]")) {
            makeNumericButton(s, button);
        } else {
            final ObjectProperty<Cal> triggerCal = determineOperand(s);
            if (triggerCal.get() != Cal.operation) {
                makeOperandButton(button, triggerCal);
            } else if ("c".equals(s)) {
                makeClearButton(button);
            } else if ("=".equals(s)) {
                makeEqualsButton(button);
            }
        }

        return button;
    }

    private ObjectProperty<Cal> determineOperand(String s) {
        final ObjectProperty<Cal> triggerOp = new SimpleObjectProperty<>(Cal.operation);
        switch (s) {
            case "+": triggerOp.set(Cal.add);      break;
            case "-": triggerOp.set(Cal.subtract); break;
            case "*": triggerOp.set(Cal.multiply); break;
            case "/": triggerOp.set(Cal.divide);   break;
            case "²": triggerOp.set(Cal.square);   break;
            case "√": triggerOp.set(Cal.squareroot);  break;
            case "^": triggerOp.set(Cal.exponent);   break;
            case "~": triggerOp.set(Cal.invert);     break;
        }
        return triggerOp;
    }

    private void makeOperandButton(Button button, final ObjectProperty<Cal> triggerCal) {
        button.setStyle("-fx-base: black;");
        button.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent actionEvent) {
                first = triggerCal.get();
            }
        });
    }

    private Button makeStandardButton(String s) {
        Button button = new Button(s);
        button.setStyle("-fx-base: pink;");
        accelerators.put(s, button);
        button.setMaxSize(Double.MAX_VALUE, Double.MAX_VALUE);
        return button;
    }

    private void makeNumericButton(final String s, Button button) {
        button.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent actionEvent) {
                if (first == Cal.operation) {
                    currentValue.set(currentValue.get() * 10 + Integer.parseInt(s));
                } else {
                    previousValue.set(currentValue.get());
                    currentValue.set(Integer.parseInt(s));
                    second = first;
                    second = Cal.operation;
                }
            }
        });
    }

    private void makeClearButton(Button button) {
        button.setStyle("-fx-base: grey;");
        button.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent actionEvent) {
                currentValue.set(0);
            }
        });
    }

    private void makeEqualsButton(Button button) {
        button.setStyle("-fx-base: green;");
        button.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent actionEvent) {
                switch (second) {
                    case add:      currentValue.set(calcEngine.addition(previousValue.get(), currentValue.get())); break;
                    case subtract: currentValue.set(calcEngine.subtraction(previousValue.get(), currentValue.get())); break;
                    case multiply: currentValue.set(calcEngine.multiplication(previousValue.get(), currentValue.get())); break;
                    case divide:   currentValue.set(calcEngine.division(previousValue.get(), currentValue.get())); break;
                    case square:   currentValue.set(calcEngine.square(previousValue.get())); break;
                    case squareroot: currentValue.set(calcEngine.squareroot(previousValue.get()));   break;
                    case exponent:  currentValue.set(calcEngine.exponent(previousValue.get(), currentValue.get())); break;
                    case invert:    currentValue.set(calcEngine.invert(previousValue.get()));   break;

                }
            }
        });
    }
}

